package core

import (
	"fmt"
	"slices"

	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/types"
	"git.akyoto.dev/cli/q/src/x86"
)

// CallExtern calls an external function.
func (f *Function) CallExtern(fn *Function, parameters []*expression.Expression) ([]types.Type, error) {
	f.DLLs = f.DLLs.Append(fn.Package, fn.Name)

	var pushedRegisters []cpu.Register

	for _, register := range x86.WindowsVolatileRegisters {
		if f.RegisterIsUsed(register) {
			f.Register(asm.PUSH, register)
			f.FreeRegister(register)
			pushedRegisters = append(pushedRegisters, register)
		}
	}

	registers := x86.WindowsInputRegisters[:len(parameters)]
	err := f.ExpressionsToRegisters(parameters, registers, fn.Input)

	if err != nil {
		return nil, err
	}

	f.DLLCall(fmt.Sprintf("%s.%s", fn.Package, fn.Name))

	for _, register := range registers {
		f.FreeRegister(register)
	}

	for _, register := range slices.Backward(pushedRegisters) {
		f.Register(asm.POP, register)
	}

	return fn.OutputTypes, nil
}
