package core

import (
	"git.akyoto.dev/cli/q/src/ast"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/types"
)

// CompileReturn compiles a return instruction.
func (f *Function) CompileReturn(node *ast.Return) error {
	if len(node.Values) == 0 {
		f.Return()
		return nil
	}

	if len(node.Values) != len(f.Output) {
		return errors.New(&errors.ReturnCountMismatch{Count: len(node.Values), ExpectedCount: len(f.Output)}, f.File, node.Values[0].Token.Position)
	}

	for i := len(node.Values) - 1; i >= 0; i-- {
		for j := range i {
			if f.UsesRegister(node.Values[j], f.CPU.Output[i]) {
				f.SaveRegister(f.CPU.Output[i])
				break
			}
		}

		typ, err := f.ExpressionToRegister(node.Values[i], f.CPU.Output[i])

		if err != nil {
			return err
		}

		if !types.Is(typ, f.Output[i].Type) {
			if f.Package == "mem" && f.Name == "alloc" {
				continue
			}

			return errors.New(&errors.TypeMismatch{Encountered: typ.Name(), Expected: f.Output[i].Type.Name(), ParameterName: "", IsReturn: true}, f.File, node.Values[i].Token.Position)
		}
	}

	f.Return()
	return nil
}
