package core

import (
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/types"
)

// ExpressionsToRegisters moves multiple expressions into the specified registers and checks that the types match with the function signature.
func (f *Function) ExpressionsToRegisters(expressions []*expression.Expression, registers []cpu.Register, input []*Input) error {
	for i := len(expressions) - 1; i >= 0; i-- {
		f.SaveRegister(registers[i])
		typ, err := f.ExpressionToRegister(expressions[i], registers[i])

		if err != nil {
			return err
		}

		if input == nil {
			continue
		}

		if !types.Is(typ, input[i].Type) {
			_, expectsPointer := input[i].Type.(*types.Pointer)

			if expectsPointer && expressions[i].Token.Kind == token.Number && expressions[i].Token.Text(f.File.Bytes) == "0" {
				continue
			}

			return errors.New(&errors.TypeMismatch{Encountered: typ.Name(), Expected: input[i].Type.Name(), ParameterName: input[i].Name}, f.File, expressions[i].Token.Position)
		}
	}

	return nil
}
