package scanner

import (
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/fs"
	"git.akyoto.dev/cli/q/src/token"
)

// scanExtern scans a block of external function declarations.
func (s *Scanner) scanExtern(file *fs.File, tokens token.List, i int) (int, error) {
	i++

	if tokens[i].Kind != token.Identifier {
		return i, errors.New(errors.ExpectedDLLName, file, tokens[i].Position)
	}

	dllName := tokens[i].Text(file.Bytes)
	i++

	if tokens[i].Kind != token.BlockStart {
		return i, errors.New(errors.MissingBlockStart, file, tokens[i].Position)
	}

	i++
	closed := false

	for i < len(tokens) {
		if tokens[i].Kind == token.Identifier {
			function, j, err := scanFunctionSignature(file, tokens, i, token.NewLine)

			if err != nil {
				return j, err
			}

			i = j
			function.Package = dllName
			function.UniqueName = dllName + "." + function.Name
			s.functions <- function
		}

		if tokens[i].Kind == token.BlockEnd {
			closed = true
			break
		}

		i++
	}

	if !closed {
		return i, errors.New(errors.MissingBlockEnd, file, tokens[i].Position)
	}

	return i, nil
}
